package com.tianyi.sync.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.tianyi.sync.mapper.master.QueryMasterMapper;
import com.tianyi.sync.mapper.slave.GroupSlaveMapper;
import com.tianyi.sync.mapper.slave2.FallGroupSlaveMapper;
import com.tianyi.sync.model.*;
import com.tianyi.sync.service.GroupSyncService;
import com.tianyi.sync.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StopWatch;


import java.util.*;
import java.util.stream.Collectors;


@Service
public class GroupSyncServiceimpl implements GroupSyncService {

    private final static Logger logger = LoggerFactory.getLogger(GroupSyncServiceimpl.class);

    private final FallGroupSlaveMapper fallGroupSlaveMapper;

    private final QueryMasterMapper queryMasterMapper;

    private final GroupSlaveMapper groupSlaveMapper;

    public GroupSyncServiceimpl(QueryMasterMapper queryMasterMapper, FallGroupSlaveMapper fallGroupSlaveMapper, GroupSlaveMapper groupSlaveMapper) {
        this.queryMasterMapper = queryMasterMapper;
        this.fallGroupSlaveMapper = fallGroupSlaveMapper;
        this.groupSlaveMapper = groupSlaveMapper;
    }

    @Override
    @DSTransactional
    public Object groupSync() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        JSONObject jsonObject = new JSONObject();
        String lastMonth = DateUtils.getYearAndMonthLast();
        //String lastMonth = "2024-12";
        String[] timeParts = lastMonth.split("-");
        String year = timeParts[0];  // 年份部分
        String month = String.valueOf(Integer.parseInt(timeParts[1]));  // 月份部分
        ParmDataModel parmDataModel = new ParmDataModel();
        parmDataModel.setPeriod(lastMonth);
        String period = parmDataModel.getPeriod();
        parmDataModel.setYear(period.substring(0, 4));
        parmDataModel.setMonth(StringUtils.startsWith(period.substring(5, 7), "0") ? StringUtils.substringAfterLast(period.substring(5, 7), "0") : period.substring(5, 7));
        logger.info("参数:{}", JSON.toJSONString(parmDataModel));
        //先清除表内数据
        fallGroupSlaveMapper.delete(parmDataModel);

        // 执行查询
        List<PlugModel> singlePlugs = groupSlaveMapper.queryPlugAccount("1");
        List<PlugModel> smallPlugs = groupSlaveMapper.queryPlugAccount("3");
        List<PlugModel> doublePlugs = groupSlaveMapper.queryPlugAccount("2");

        // 创建一个Map来存储所有的单边抵映射关系
        Map<String, Map<String, List<String>>> singleMapping = new HashMap<>();
        Map<String, Map<String, List<String>>> smallMapping = new HashMap<>();
        Map<String, Map<String, List<String>>> doubleMapping = new HashMap<>();

        // 处理查询结果
        processPlugList(singlePlugs, singleMapping);
        processPlugList(smallPlugs, smallMapping);
        processPlugList(doublePlugs, doubleMapping);

        // 对 singleMapping 中的每个 accountList 按照 pluggroup 排序
//        for (Map<String, List<String>> sideMapping : singleMapping.values()) {
//            for (List<String> accountList : sideMapping.values()) {
//                // 这里是对 accountList 进行排序，假设每个 PlugModel 的 account 对应的 pluggroup
//                accountList.sort(new Comparator<String>() {
//                    @Override
//                    public int compare(String account1, String account2) {
//                        // 获取对应 account 的 pluggroup
//                        String pluggroup1 = getPlugGroupForAccount(account1, singlePlugs); // 获取 account1 对应的 pluggroup
//                        String pluggroup2 = getPlugGroupForAccount(account2, singlePlugs); // 获取 account2 对应的 pluggroup
//                        return pluggroup1.compareTo(pluggroup2); // 按 pluggroup 排序
//                    }
//                });
//            }
//        }


        // 单边抵消c3特殊处理
        Map<String, String> C3_02_value = new HashMap<>();
        C3_02_value.put("Inc99", "Dec99");
        C3_02_value.put("PInc02", "PDec0201");
        C3_02_value.put("Inc02", "Dec0201");
        C3_02_value.put("[None]", "[None]");
        C3_02_value.put("MovBeg", "[None]");

        //查询全集团数据
        List<GroupDataSyncModel> dataSyncModelList = queryMasterMapper.queryGroupCurrentMonth(parmDataModel);
        //查询科目表
        List<AccountNameModel> accountNameList = queryMasterMapper.queryAccountName();
        Map<String, String> accountNameMap = new HashMap<>();
        for (AccountNameModel accountNameModel : accountNameList) {
            accountNameMap.put(accountNameModel.getCode(), accountNameModel.getName());
        }
        accountNameMap.put("NB2221028", "各单位上交增值税");
        accountNameMap.put("NB41040102", "调整年初未分配利润");


        //TODO  处理后的数据放这里
        List<GroupDataSyncModel> dataSyncList = new ArrayList<>();

        //单边抵
        // 遍历singleMapping
        for (Map.Entry<String, Map<String, List<String>>> singlMap : singleMapping.entrySet()) {
            String singlMapKey = singlMap.getKey();
            Map<String, List<String>> plugData = singlMap.getValue();
            List<String> singlePlug01 = plugData.get("01");
            List<String> singlePlug02 = plugData.get("02");
            List<GroupDataSyncModel> groupSingle01List = queryMasterMapper.queryGroupSingle(lastMonth, singlePlug01);
            if ("plug16".equals(singlMapKey)) {
                List<GroupDataSyncModel> queryMovBegList = queryMasterMapper.queryMovBegData(lastMonth);
                groupSingle01List.addAll(queryMovBegList);
            }
            for (GroupDataSyncModel model : groupSingle01List) {
                String c1 = model.getC1();
                String c3 = model.getC3();
                String entity = model.getEntity();
                String icp = model.getICP();
                // 根据 key 和 c3、c1 的组合条件，选择合适的处理
                if ("plug18".equals(singlMapKey) && ("Inc99".equals(c3) || "PInc02".equals(c3) || "Inc02".equals(c3))) {
                    List<GroupDataSyncModel> modelList = processSyncModel(singlMapKey, accountNameMap, model, singlePlug01, singlePlug02, C3_02_value.get(model.getC3()), null, null);
                    dataSyncList.addAll(modelList);
                } else if ("plug16".equals(singlMapKey) && ("[None]".equals(c3) || "MovBeg01".equals(c3))) {
                    List<GroupDataSyncModel> modelList = processSyncModel(singlMapKey, accountNameMap, model, singlePlug01, singlePlug02, C3_02_value.get(model.getC3()), null, null);
                    dataSyncList.addAll(modelList);
                } else if ("plug29".equals(singlMapKey) && "[None]".equals(c3) && "A01".equals(entity) && "[ICP_None]".equals(icp)) {
                    List<GroupDataSyncModel> modelList = processSyncModel(singlMapKey, accountNameMap, model, singlePlug01, singlePlug02, C3_02_value.get(model.getC3()), null, null);
                    dataSyncList.addAll(modelList);
                } else if ("plug25".equals(singlMapKey) && "[None]".equals(c3) && !"FM9903".equals(c1) && !"FM9906".equals(c1)) {
                    List<GroupDataSyncModel> modelList = processSyncModel(singlMapKey, accountNameMap, model, singlePlug01, singlePlug02, C3_02_value.get(model.getC3()), model.getICP(), model.getIcpname());
                    dataSyncList.addAll(modelList);
                } else if ("[None]".equals(c3) && !("plug18".equals(singlMapKey) || "plug16".equals(singlMapKey) || "plug29".equals(singlMapKey) || "plug25".equals(singlMapKey))) {
                    List<GroupDataSyncModel> modelList = processSyncModel(singlMapKey, accountNameMap, model, singlePlug01, singlePlug02, C3_02_value.get(model.getC3()), model.getICP(), model.getIcpname());
                    dataSyncList.addAll(modelList);
                } else {
                    continue;
                }
            }
        }


        //存储以 entity-icp为键的元素列表
        Map<String, List<GroupDataSyncModel>> entityIcpCodeMap = new HashMap<>();
        for (GroupDataSyncModel item : dataSyncModelList) {
            String entityIcpKey = item.getEntity() + "-" + item.getICP();
            entityIcpCodeMap.computeIfAbsent(entityIcpKey, k -> new ArrayList<>()).add(item);
        }
        //以小抵消
        for (Map.Entry<String, Map<String, List<String>>> smallMap : smallMapping.entrySet()) {
            String plugAccount = smallMap.getKey();
            //获取plug两侧科目
            Map<String, List<String>> plugDataMap = smallMap.getValue();
            List<String> plugSide01 = plugDataMap.get("01");
            List<String> plugSinde02 = plugDataMap.get("02");
            //按主体客商执行
            for (Map.Entry<String, List<GroupDataSyncModel>> entry : entityIcpCodeMap.entrySet()) {
                String entityIcpKey = entry.getKey();
                List<GroupDataSyncModel> entityIcpList = entry.getValue();
                // 筛选符合条件的集合
                List<GroupDataSyncModel> filteredList01 = filterEntityList(entityIcpList, plugSide01, plugAccount);
                // 分析键的形式：entityCode-icpCode
                String[] parts = entityIcpKey.split("-");
                String entityCode = parts[0];
                String icpCode = parts[1];
                List<GroupDataSyncModel> icpEntityList = entityIcpCodeMap.get(icpCode + "-" + entityCode);
                // 过滤普通和反转后的客商02侧列表
                List<GroupDataSyncModel> filteredList02 = filterEntityList(icpEntityList, plugSinde02, plugAccount);
                // 计算 money01 和 money02
                Double money01 = calculateTotalMoney(filteredList01);
                Double money02 = calculateTotalMoney(filteredList02);
                if (money01 == 0 || money02 == 0) {
                    continue;
                }
                if (Double.compare(money01, money02) > 0) {
                    // 如果 money01 > money02
                    smallGroupDataSyncList(plugAccount, filteredList01, filteredList02, dataSyncList, money02, "1");
                    filteredList02.forEach(item -> {
                        String money = processMoney(item.getMoney());
                        item.setMoney2(money);
                        item.setMoney(null);
                        item.setC4("Interadj");
                        item.setPlugAccount(plugAccount);
                    });
                    dataSyncList.addAll(filteredList02);
                } else if (Double.compare(money01, money02) < 0) {
                    // 如果 money01 < money02
                    filteredList01.forEach(item -> {
                        String money = processMoney(item.getMoney());
                        item.setMoney(money);
                        item.setC4("Interadj");
                        item.setPlugAccount(plugAccount);
                    });
                    dataSyncList.addAll(filteredList01);
                    smallGroupDataSyncList(plugAccount, filteredList02, filteredList01, dataSyncList, money01, "2");
                } else {
                    // 如果 money01 == money02
                    filteredList01.forEach(item -> {
                        String money = processMoney(item.getMoney());
                        item.setMoney(money);
                        item.setC4("Interadj");
                        item.setPlugAccount(plugAccount);
                    });
                    dataSyncList.addAll(filteredList01);
                    filteredList02.forEach(item -> {
                        String money2 = processMoney(item.getMoney());
                        item.setMoney2(money2);
                        item.setMoney(null);
                        item.setC4("Interadj");
                        item.setPlugAccount(plugAccount);
                    });
                    dataSyncList.addAll(filteredList02);
                }
                //空行
                GroupDataSyncModel emptyModel = new GroupDataSyncModel();
                emptyModel.setYear(year);
                emptyModel.setMonth(month);
                emptyModel.setEntity(entityCode);
                emptyModel.setICP(icpCode);
                emptyModel.setPlugAccount(plugAccount);
                dataSyncList.add(emptyModel);
            }
        }

        //双边抵消
        //查询164库内数据
        for (Map.Entry<String, Map<String, List<String>>> stringMapEntry : doubleMapping.entrySet()) {
            String plugAccount = stringMapEntry.getKey();
            //plug参数
            boolean isReverse = (plugAccount.equals("plug11") || plugAccount.equals("plug12") || plugAccount.equals("plug13"));
            String destC1 = (plugAccount != null && plugAccount.equals("plug12")) ? "FC01" : "[None]";
            String destC2 = (plugAccount != null && (plugAccount.equals("plug12") || plugAccount.equals("plug17")
                    || plugAccount.equals("plug21") || plugAccount.equals("plug22"))) ? "SL02" : "[None]";
            destC2 = (plugAccount != null && plugAccount.equals("plug25")) ? "SL01" : destC2;
            //获取plug两侧科目
            Map<String, List<String>> plugDataMap = stringMapEntry.getValue();
            List<String> plugSide01 = plugDataMap.get("01");
            List<String> plugSinde02 = plugDataMap.get("02");
            //按主体客商执行
            for (Map.Entry<String, List<GroupDataSyncModel>> entry : entityIcpCodeMap.entrySet()) {
                String entityIcpKey = entry.getKey();
                List<GroupDataSyncModel> entityIcpList = entry.getValue();
                // 筛选符合条件的集合
                List<GroupDataSyncModel> filteredList01 = filterEntityList(entityIcpList, plugSide01, plugAccount);
                // 分析键的形式：entityCode-icpCode
                String[] parts = entityIcpKey.split("-");
                String entityCode = parts[0];
                String icpCode = parts[1];
                List<GroupDataSyncModel> icpEntityList = entityIcpCodeMap.get(icpCode + "-" + entityCode);
                // 过滤普通和反转后的客商02侧列表
                List<GroupDataSyncModel> filteredList02 = filterEntityList(icpEntityList, plugSinde02, plugAccount);
                // 如果两个集合都为空，跳出循环
                if (filteredList01.isEmpty() && filteredList02.isEmpty()) {
                    continue;
                }
                if (plugAccount.equals("plug26")) {
                    // 根据 c1 对 filteredList01 和 filteredList02 进行分组
                    Map<String, List<GroupDataSyncModel>> groupedByC1List01 = filteredList01.stream()
                            .collect(Collectors.groupingBy(GroupDataSyncModel::getC1));

                    Map<String, List<GroupDataSyncModel>> groupedByC1List02 = filteredList02.stream()
                            .collect(Collectors.groupingBy(GroupDataSyncModel::getC1));
                    // 计算并打印每个 c1 的 total money
                    Set<String> allC1Keys = new HashSet<>();
                    allC1Keys.addAll(groupedByC1List01.keySet());
                    allC1Keys.addAll(groupedByC1List02.keySet());
                    for (String c1 : allC1Keys) {
                        // 计算 money01 和 money02
                        List<GroupDataSyncModel> list01 = groupedByC1List01.get(c1);
                        List<GroupDataSyncModel> list02 = groupedByC1List02.get(c1);
                        Double money01 = calculateTotalMoney(list01);
                        Double money02 = calculateTotalMoney(list02);
                        filteredList01.forEach(item -> {
                            String money = processMoney(item.getMoney());
                            item.setMoney(money);
                            item.setC4("Interadj");
                            item.setPlugAccount(plugAccount);
                        });
                        dataSyncList.addAll(list01);
                        list02.forEach(item -> {
                            String money2 = processMoney(item.getMoney());
                            item.setMoney2(money2);
                            item.setMoney(null);
                            item.setC4("Interadj");
                        });
                        dataSyncList.addAll(list02);
                        plugAccountDataSync(money01,money02,isReverse,dataSyncList, plugAccount,  year,  month, destC1, destC2, entityCode, icpCode);
                        //空行
                        GroupDataSyncModel emptyModel = new GroupDataSyncModel();
                        emptyModel.setYear(year);
                        emptyModel.setMonth(month);
                        emptyModel.setEntity(entityCode);
                        emptyModel.setICP(icpCode);
                        dataSyncList.add(emptyModel);
                    }
                } else {
                    // 计算 money01 和 money02
                    Double money01 = calculateTotalMoney(filteredList01);
                    Double money02 = calculateTotalMoney(filteredList02);
                    filteredList01.forEach(item -> {
                        String money = processMoney(item.getMoney());
                        item.setMoney(money);
                        item.setC4("Interadj");
                        item.setPlugAccount(plugAccount);
                    });
                    dataSyncList.addAll(filteredList01);
                    filteredList02.forEach(item -> {
                        String money2 = processMoney(item.getMoney());
                        item.setMoney2(money2);
                        item.setMoney(null);
                        item.setC4("Interadj");
                        item.setPlugAccount(plugAccount);
                    });
                    dataSyncList.addAll(filteredList02);
                    plugAccountDataSync(money01,money02,isReverse,dataSyncList, plugAccount,  year,  month, destC1, destC2, entityCode, icpCode);
                    GroupDataSyncModel emptyModel = new GroupDataSyncModel();
                    emptyModel.setEntity(entityCode);
                    emptyModel.setICP(icpCode);
                    emptyModel.setYear(year);
                    emptyModel.setMonth(month);
                    emptyModel.setPlugAccount(plugAccount);
                    dataSyncList.add(emptyModel);
                }
            }
        }

        //plug19主营业务用
        List<GroupDataSyncModel> plugDataList = plug19DataSync(entityIcpCodeMap, year, month);
        dataSyncList.addAll(plugDataList);
        //数量抵消
        List<GroupDataSyncModel> qtySyncList = addAccountData(dataSyncModelList, year, month);
        dataSyncList.addAll(qtySyncList);

        // 2. 为每个元素加上顺序值，顺序从 1 开始
        for (int i = 0; i < dataSyncList.size(); i++) {
            GroupDataSyncModel model = dataSyncList.get(i);
            model.setSortCode(i + 1);  // 设置顺序值，顺序从 1 开始
        }

        if (CollectionUtils.isEmpty(dataSyncList)) {
            logger.info("unexpected event {}:", "未查询到数据!");
            jsonObject.put("description", "未查询到数据!");
            return jsonObject;
        }
        List<GroupFallDataModel> groupFallDataModelList = dataSyncList.stream().map(v -> {
            GroupFallDataModel groupFallDataModel = new GroupFallDataModel();
            BeanUtils.copyProperties(v, groupFallDataModel);
            groupFallDataModel.setAccountcode(v.getAccount());
            groupFallDataModel.setEntitycode(v.getEntity());
            groupFallDataModel.setOrivalue(v.getMoney());
            groupFallDataModel.setOrivalue2(v.getMoney2());
            groupFallDataModel.setICPcode(v.getICP());
            groupFallDataModel.setICPname(v.getIcpname());
            groupFallDataModel.setYtenantId("dmrn1j0i");
            groupFallDataModel.setPlugaccount(v.getPlugAccount());
            return groupFallDataModel;
        }).collect(Collectors.toList());

        //数据落到96数据库内
        fallGroupSlaveMapper.addAll(groupFallDataModelList);
        jsonObject.put("msg", "数据同步成功!");
        jsonObject.put("code", 200);
        stopWatch.stop();
        logger.info("执行时间:{}", stopWatch.getLastTaskTimeMillis());
        return jsonObject;

    }

    // 处理 PlugModel 列表并将数据填充到对应的 Map 中
    private void processPlugList(List<PlugModel> plugList, Map<String, Map<String, List<String>>> mapping) {
        for (PlugModel plug : plugList) {
            String plugAccount = plug.getPlugaccount();
            String plugSide = plug.getPlugside();
            String account = plug.getAccount();

            if (!mapping.containsKey(plugAccount)) {
                mapping.put(plugAccount, new HashMap<>());
            }

            Map<String, List<String>> sideMapping = mapping.get(plugAccount);

            if (!sideMapping.containsKey(plugSide)) {
                sideMapping.put(plugSide, new ArrayList<>());
            }

            List<String> accountList = sideMapping.get(plugSide);

            accountList.add(account);
        }
    }

    //单边抵消科目排序
    private String getPlugGroupForAccount(String account, List<PlugModel> plugs) {
        for (PlugModel plug : plugs) {
            if (plug.getAccount().equals(account)) {
                return plug.getPluggroup(); // 返回对应的 pluggroup
            }
        }
        return "";  // 默认返回空字符串
    }

    // 单边抵消方法处理同步数据
    private List<GroupDataSyncModel> processSyncModel(String plugAccount, Map<String, String> accountNameMap, GroupDataSyncModel model, List<String> singlePlug01, List<String> singlePlug02,
                                                      String c3Value, String entity, String entityName) {
        List<GroupDataSyncModel> dataSyncList = new ArrayList<>();
        // 获取对应的 account02
        String account02Value = null;
        if ("plug16".equals(plugAccount)) {
            int index = singlePlug01.indexOf(model.getAccount());
            account02Value = singlePlug02.get(index + 1);
        } else {
            int index = singlePlug01.indexOf(model.getAccount());
            account02Value = singlePlug02.get(index);
        }
        //02侧数据
        GroupDataSyncModel syncModel = new GroupDataSyncModel();
        BeanUtils.copyProperties(model, syncModel);
        // 将 syncModel 中的 money 值赋给 money2
        if ("plug16".equals(plugAccount) && "MovBeg01".equals(model.getC3())) {
            syncModel.setMoney2(model.getMoney());
        } else {
            String money = processMoney(model.getMoney());
            syncModel.setMoney2(money);
        }
        syncModel.setMoney(null);
        syncModel.setAccount(account02Value);
        syncModel.setAccountname(accountNameMap.get(account02Value));
        syncModel.setC3(c3Value);
        // 如果 entity 和 entityName 有值，进行设置
        if (entity != null) {
            syncModel.setEntity(entity);
            syncModel.setEntityname(entityName);
            syncModel.setICP(model.getEntity());
            syncModel.setIcpname(model.getEntityname());
        }
        // 设置 C4 的值
        syncModel.setC4("interadj");
        syncModel.setPlugAccount(plugAccount);

        //对原始值进行处理
        if ("2221028".equals(model.getAccount())) {
            model.setAccount("NB2221028");
        }
        String value = processMoney(model.getMoney());
        model.setMoney(value);
        model.setC4("interadj");
        model.setPlugAccount(plugAccount);
        dataSyncList.add(model);
        //对plug19、plug25处理
        dataSyncList.add(syncModel);
        if ("plug19".equals(plugAccount) || "plug25".equals(plugAccount)) {
            GroupDataSyncModel syncIncModel = new GroupDataSyncModel();
            BeanUtils.copyProperties(syncModel, syncIncModel);
            syncIncModel.setC3("Inc99");
            syncIncModel.setPlugAccount(plugAccount);
            dataSyncList.add(syncIncModel);
        }
        //处理plug16的MovBeg01
        if ("plug16".equals(plugAccount) && "MovBeg01".equals(model.getC3())) {
            int index = singlePlug01.indexOf(model.getAccount());
            String account02 = singlePlug02.get(index);
            //02侧数据
            GroupDataSyncModel MovBegModel = new GroupDataSyncModel();
            BeanUtils.copyProperties(model, MovBegModel);
            // 将 syncModel 中的 money 值赋给 money2
            String money = processMoney(model.getMoney());
            MovBegModel.setMoney2(money);
            MovBegModel.setMoney(null);
            MovBegModel.setAccount(account02);
            MovBegModel.setAccountname(accountNameMap.get(account02));
            MovBegModel.setC3(c3Value);
            MovBegModel.setC4("interadj");
            MovBegModel.setPlugAccount(plugAccount);
            dataSyncList.add(MovBegModel);
        }
        //添加空行
        GroupDataSyncModel emptyModel = new GroupDataSyncModel();
        emptyModel.setYear(model.getYear());
        emptyModel.setMonth(model.getMonth());
        emptyModel.setEntity(model.getEntity());
        emptyModel.setICP(model.getICP());
        emptyModel.setPlugAccount(plugAccount);
        dataSyncList.add(emptyModel);
        return dataSyncList;
    }

    // 过滤实体列表
    private List<GroupDataSyncModel> filterEntityList(List<GroupDataSyncModel> entityIcpList, List<String> plug, String plugAccount) {
        // 如果 entityIcpList 或 plug 为 null，返回空列表或抛出异常
        if (entityIcpList == null || plug == null) {
            return Collections.emptyList();  // 返回空列表
        }

        return entityIcpList.stream()
                .filter(item -> plug.contains(item.getAccount())
                        && "[None]".equals(item.getC3())
                        && ("plug25".equals(plugAccount) ?
                        ("FM9903".equals(item.getC1()) || "FM9906".equals(item.getC1())) :
                        true)
                        && ("plug26".equals(plugAccount) ?
                        ("SM01".equals(item.getC1()) || "SM02".equals(item.getC1())) :
                        true))
                .collect(Collectors.toList());
    }

    // 通用的求和方法
    private Double calculateTotalMoney(List<GroupDataSyncModel> groupDataList) {
        // 如果集合为空，直接返回 0.0
        if (groupDataList == null || groupDataList.isEmpty()) {
            return 0.0;
        }
        return groupDataList.stream().mapToDouble(data -> {
            try {
                return Double.parseDouble(data.getMoney());
            } catch (NumberFormatException e) {
                return 0.0; // 如果无法转换为 double，返回 0.0
            }
        }).sum();
    }


    private void smallGroupDataSyncList(String plugAccount, List<GroupDataSyncModel> filteredList1, List<GroupDataSyncModel> filteredList2, List<GroupDataSyncModel> dataSyncList, double money, String type) {

        // 排序操作只需做一次
        filteredList1.sort((g1, g2) -> g1.getAccount().compareTo(g2.getAccount()));
        filteredList2.sort((g1, g2) -> g1.getAccount().compareTo(g2.getAccount()));
        // 遍历filteredList1，并在filteredList2中查找匹配的元素
        for (int i = filteredList1.size() - 1; i >= 0; i--) {
            GroupDataSyncModel groupData1 = filteredList1.get(i);
            for (int j = filteredList2.size() - 1; j >= 0; j--) {
                GroupDataSyncModel groupData2 = filteredList2.get(j);
                if (groupData1.getMoney() == groupData2.getMoney()) {
                    // 将匹配的值添加到dataSyncList中
                    if ("2".equals(type)) {
                        String value2 = processMoney(groupData1.getMoney());
                        groupData1.setMoney2(value2);
                        groupData1.setMoney(null);
                    } else {
                        String value = processMoney(groupData1.getMoney());
                        groupData1.setMoney(value);
                    }
                    groupData1.setC4("Interadj");
                    groupData1.setPlugAccount(plugAccount);
                    dataSyncList.add(groupData1);

                    // 从filteredList1和filteredList2中移除已匹配的元素
                    filteredList1.remove(i);
                    filteredList2.remove(j);
                    money -= Double.parseDouble(groupData1.getMoney());

                    // 跳出内层循环，继续处理filteredList2中的下一个元素
                    break;
                }
            }
        }
        if (money >= 0) {
            for (GroupDataSyncModel model : filteredList1) {
                double accountMoney = Double.parseDouble(model.getMoney());

                if (accountMoney <= money) {
                    if ("2".equals(type)) {
                        String value2 = processMoney(model.getMoney());
                        model.setMoney2(value2);
                        model.setMoney(null);
                    } else {
                        String value = processMoney(model.getMoney());
                        model.setMoney(value);
                    }
                    model.setC4("Interadj");
                    model.setPlugAccount(plugAccount);
                    dataSyncList.add(model);
                    money -= accountMoney;
                    if (money == 0) {
                        break;
                    }
                } else {
                    GroupDataSyncModel syncModel = new GroupDataSyncModel();
                    BeanUtils.copyProperties(model, syncModel);
                    syncModel.setC4("interadj");
                    if ("2".equals(type)) {
                        syncModel.setMoney2(String.valueOf(-money));
                        syncModel.setMoney(null);
                    } else {
                        syncModel.setMoney(String.valueOf(-money));
                    }
                    syncModel.setPlugAccount(plugAccount);
                    dataSyncList.add(syncModel);
                    money = 0; // 直接设置 money02 为 0，避免不必要的计算
                    break;
                }
            }
        }
    }

    private void plugAccountDataSync(Double money01,Double money02,Boolean isReverse,List<GroupDataSyncModel> dataSyncList, String plugAccount, String year, String month,
                                     String destC1, String destC2, String entityCode,
                                     String icpCode){
        if (Double.compare(money01, money02) != 0) {
            if (isReverse) {
                if (money01.compareTo(0.0) != 0 && money02.compareTo(0.0) == 0) {
                    addSyncModel(dataSyncList, plugAccount, year, month, destC1, destC2, entityCode, icpCode, money01 - money02, Double.NaN);
                } else if (money01.compareTo(0.0) == 0 && money02.compareTo(0.0) != 0) {
                    addSyncModel(dataSyncList, plugAccount, year, month, destC1, destC2, icpCode, entityCode, Double.NaN, money01 - money02);
                } else {
                    addSyncModel(dataSyncList, plugAccount, year, month, destC1, destC2, entityCode, icpCode, money01 - money02, Double.NaN);
                }
            } else {
                if (money01.compareTo(0.0) != 0 && money02.compareTo(0.0) == 0) {
                    addSyncModel(dataSyncList, plugAccount, year, month, destC1, destC2, entityCode, icpCode, money02 - money01, Double.NaN);
                } else if (money01.compareTo(0.0) == 0 && money02.compareTo(0.0) != 0) {
                    addSyncModel(dataSyncList, plugAccount, year, month, destC1, destC2, icpCode, entityCode, Double.NaN, money02 - money01);
                } else {
                    addSyncModel(dataSyncList, plugAccount, year, month, destC1, destC2, entityCode, icpCode, money02 - money01, Double.NaN);
                }
            }
        }
    }

    private void addSyncModel(List<GroupDataSyncModel> dataSyncList, String plugAccount, String year, String month,
                              String destC1, String destC2, String entityCode,
                              String icpCode, Double money, Double money2) {
        //获取科目名称
        List<EntityNameModel> entityNameList = queryMasterMapper.queryEntityName();
        Map<String, String> entityNameMap = new HashMap<>();
        for (EntityNameModel entityNameModel : entityNameList) {
            entityNameMap.put(entityNameModel.getCode(), entityNameModel.getName());
        }
        GroupDataSyncModel syncModel = new GroupDataSyncModel();
        syncModel.setYear(year);
        syncModel.setMonth(month);
        syncModel.setAccount(plugAccount);
        syncModel.setICP("[ICP_None]");
        syncModel.setC1(destC1);
        syncModel.setC2(destC2);
        syncModel.setC3("[None]");
        syncModel.setC4("Interadj");
        syncModel.setEntity(entityCode);
        syncModel.setEntityname(entityNameMap.get(entityCode.replace("A", "")));
        syncModel.setICP(icpCode);
        syncModel.setIcpname(entityNameMap.get(icpCode.replace("A", "")));
        if (!Double.isNaN(money)) {
            syncModel.setMoney(String.valueOf(money));
        }
        if (!Double.isNaN(money2)) {
            syncModel.setMoney2(String.valueOf(money2));
        }
        syncModel.setPlugAccount(plugAccount);
        dataSyncList.add(syncModel);
    }

    /**
     * plug19 部分 主营业务金额抵消
     */
    private List<GroupDataSyncModel> plug19DataSync(Map<String, List<GroupDataSyncModel>> entityIcpCodeMap, String year, String month) {


        String[] accountList = {"6001001001001001001001", "6001001001001001001002", "6001001001002001003001", "6001001001002001003002", "6001001001003001001001", "6001001001003001001002"};

        String[] c1List = {"MT01", "MT0201", "MT0202", "MT0203", "MT020401", "MT020402", "MT0301", "MT0302", "MT0303"};
        //存储处理好的数据
        List<GroupDataSyncModel> plug19DataList = new ArrayList<>();

        for (Map.Entry<String, List<GroupDataSyncModel>> entry : entityIcpCodeMap.entrySet()) {
            String entityIcp = entry.getKey();
            String[] parts = entityIcp.split("-");
            String entity = parts[0];
            String icp = parts[1];
            List<GroupDataSyncModel> models = entry.getValue();
            //处理第一部分数据
            List<GroupDataSyncModel> data = models.stream()
                    .filter(e -> "[None]".equals(e.getC3()) && Arrays.asList(accountList).contains(e.getAccount()) && Arrays.asList(c1List).contains(e.getC1()))
                    .collect(Collectors.toList());
            mainBusinessData(data, plug19DataList, "1405001001", "Inc99", "Interadj");

            //处理第二部分数据
            List<GroupDataSyncModel> data2 = models.stream()
                    .filter(e -> "[None]".equals(e.getC3()) && "6001002006001001".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data2, plug19DataList, "1405002001", "Inc99", "Interadj");

            //处理第三部分数据
            List<GroupDataSyncModel> data3 = models.stream()
                    .filter(e -> "[None]".equals(e.getC3()) && "6001002006002001".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data3, plug19DataList, "1405003001", "Inc99", "Interadj");

            //处理第四部分数据
            List<GroupDataSyncModel> data4 = models.stream()
                    .filter(e -> "[None]".equals(e.getC3()) && "6001002006003001".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data4, plug19DataList, "1405003044", "Inc99", "Interadj");

            //处理第五部分数据
            List<GroupDataSyncModel> data5 = models.stream()
                    .filter(e -> "FM0101".equals(e.getC3()) && "[None]".equals(e.getC3()) && "6001002001001".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data5, plug19DataList, "1405010", "Inc99", "Interadj");

            //处理第六部分数据
            List<GroupDataSyncModel> data6 = models.stream()
                    .filter(e -> "FM0102".equals(e.getC3()) && "[None]".equals(e.getC3()) && "6001002001001".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data6, plug19DataList, "1405011001", "Inc99", "Interadj");

            //处理第七部分数据
            List<GroupDataSyncModel> data7 = models.stream()
                    .filter(e -> "FM99".equals(e.getC3()) && "[None]".equals(e.getC3()) && "6001002099001".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data7, plug19DataList, "1405013", "Inc99", "Interadj");

            //处理第八部分数据
            List<GroupDataSyncModel> data8 = models.stream()
                    .filter(e -> "[None]".equals(e.getC3()) && "6001001001002001005".equals(e.getAccount()))
                    .collect(Collectors.toList());
            mainBusinessData(data8, plug19DataList, "1413001002001", "Inc99", "Interadj");
            // 检查所有的 data 列表是否都不为空
            if (!data.isEmpty() || !data2.isEmpty() || !data3.isEmpty() || !data4.isEmpty()
                    || !data5.isEmpty() || !data6.isEmpty() || !data7.isEmpty() || !data8.isEmpty()) {
                // 所有列表都不为空时才添加 emptyModel
                GroupDataSyncModel emptyModel = new GroupDataSyncModel();
                emptyModel.setYear(year);
                emptyModel.setMonth(month);
                emptyModel.setEntity(entity);
                emptyModel.setICP(icp);
                emptyModel.setPlugAccount("plug19主营业务");
                plug19DataList.add(emptyModel);
            }

        }

        return plug19DataList;
    }

    public void mainBusinessData(List<GroupDataSyncModel> groupDataSyncModels, List<GroupDataSyncModel> plug19DataList, String account, String c3, String c4) {
        for (GroupDataSyncModel groupDataSyncModel : groupDataSyncModels) {
            GroupDataSyncModel dataSyncModel = new GroupDataSyncModel();
            BeanUtils.copyProperties(groupDataSyncModel, dataSyncModel);
            String money = processMoney(groupDataSyncModel.getMoney());
            dataSyncModel.setMoney(money);
            dataSyncModel.setAccount(account);
            dataSyncModel.setC3(c3);
            dataSyncModel.setC4(c4);
            dataSyncModel.setPlugAccount("plug19主营业务");
            plug19DataList.add(dataSyncModel);
        }
    }

    private List<GroupDataSyncModel> addAccountData(List<GroupDataSyncModel> groupDataSyncModels, String year, String month) {


        String[] accountList = {"6001001001001001001001", "6001001001001001001002", "6001001001002001003001", "6001001001002001003002", "6001001001003001001001", "6001001001003001001002"};

        String[] c1List = {"MT01", "MT0201", "MT0202", "MT0203", "MT020401", "MT020402", "MT0301", "MT0302", "MT0303"};
        //存储处理好的数据
        List<GroupDataSyncModel> insertData = new ArrayList<>();
        //处理第一部分数据
        List<GroupDataSyncModel> data = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && Arrays.asList(accountList).contains(e.getAccount()) && Arrays.asList(c1List).contains(e.getC1()))
                .collect(Collectors.toList());

        dealData(data, insertData, "1405001001", "QtyInc", "Interadj");

        //处理第二部分数据
        List<GroupDataSyncModel> secondData = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001002003001".equals(e.getAccount()))
                .collect(Collectors.toList());

        dealData(secondData, insertData, "6401002003001", "QtyInc", "Interadj");
        //处理第四对数据  第一对
        List<GroupDataSyncModel> data1 = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001002006001001".equals(e.getAccount()))
                .collect(Collectors.toList());

        dealData(data1, insertData, "1405002001", "QtyInc", "Interadj");
        //处理第四对数据  第二对
        List<GroupDataSyncModel> data2 = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001002006002001".equals(e.getAccount()))
                .collect(Collectors.toList());

        dealData(data2, insertData, "1405003001", "QtyInc", "Interadj");
        //处理第四对数据  第三对
        List<GroupDataSyncModel> data3 = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001002006003001".equals(e.getAccount()))
                .collect(Collectors.toList());
        dealData(data3, insertData, "1405003044", "QtyInc", "Interadj");

        //处理第四对数据  第四对
        List<GroupDataSyncModel> data4 = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001001001002001005".equals(e.getAccount()))
                .collect(Collectors.toList());
        dealData(data4, insertData, "1413001002001", "QtyInc", "Interadj");


        //存储只需要更换部分的数据

        // acc 6001002001001  c1  FM0101   c3  Qty
        //换成 acc 1405010  c1 FM0101  c3 QtyInc c4  Interadj
        List<GroupDataSyncModel> personalData = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001002001001".equals(e.getAccount()))
                .collect(Collectors.toList());
        // 使用 groupingBy 根据 entity 和 icp 进行分组，键是组合的字符串
        Map<String, List<GroupDataSyncModel>> groupedData = personalData.stream()
                .collect(Collectors.groupingBy(
                        e -> e.getEntity() + "-" + e.getICP()  // 合并 entity 和 icp 为一个字符串
                ));
        groupedData.forEach((key, models) -> {
            replaceData(models, insertData, "6001002001001", "FM0101", "Qty", "1405010", "FM0101", "FM0102", "Qty", "1405011001", "FM99", "QtyInc", "Interadj", year, month);
        });


        List<GroupDataSyncModel> personalData1 = groupDataSyncModels.stream()
                .filter(e -> "Qty".equals(e.getC3()) && "6001002099001".equals(e.getAccount()))
                .collect(Collectors.toList());
        Map<String, List<GroupDataSyncModel>> groupedData2 = personalData1.stream()
                .collect(Collectors.groupingBy(
                        e -> e.getEntity() + "-" + e.getICP()  // 合并 entity 和 icp 为一个字符串
                ));
        groupedData2.forEach((key, models) -> {
            replaceData(models, insertData, "6001002099001", "FM99", "Qty", "1405013", "FM99", null, null, null, null, "QtyInc", "Interadj", year, month);
        });
        return insertData;
    }

    //统一处理数据,为了方便插入数据

    /**
     * 数据本身与处理后的数据都需要插入
     */
    public void dealData(List<GroupDataSyncModel> groupDataSyncModels, List<GroupDataSyncModel> insertData, String account, String c3, String c4) {
        for (GroupDataSyncModel groupDataSyncModel : groupDataSyncModels) {
            groupDataSyncModel.setPlugAccount("plug19数量");
            String money = processMoney(groupDataSyncModel.getMoney());
            //先放一个
            groupDataSyncModel.setMoney(money);
            groupDataSyncModel.setC4("Interadj");
            insertData.add(groupDataSyncModel);
            //02侧
            GroupDataSyncModel groupDataSyncModel1 = new GroupDataSyncModel();
            BeanUtils.copyProperties(groupDataSyncModel, groupDataSyncModel1);
            groupDataSyncModel1.setAccount(account);
            groupDataSyncModel1.setC3(c3);
            groupDataSyncModel1.setC4(c4);
            groupDataSyncModel1.setMoney2(money);
            groupDataSyncModel1.setMoney(null);
            groupDataSyncModel.setC4("Interadj");
            groupDataSyncModel1.setPlugAccount("plug19数量");
            //修改后再放一个
            insertData.add(groupDataSyncModel1);
            //添加空行
            GroupDataSyncModel emptyModel = new GroupDataSyncModel();
            emptyModel.setYear(groupDataSyncModel.getYear());
            emptyModel.setMonth(groupDataSyncModel.getMonth());
            emptyModel.setEntity(groupDataSyncModel.getEntity());
            emptyModel.setICP(groupDataSyncModel.getICP());
            emptyModel.setPlugAccount("plug19数量");
            insertData.add(emptyModel);
        }
    }


    /**
     * 只需要插入替换后的数据
     */
    public void replaceData(List<GroupDataSyncModel> groupDataSyncModels, List<GroupDataSyncModel> insertData, String account, String c1, String c3, String replaceAcc, String replaceC1,
                            String c1_02, String c3_02, String replaceAcc2, String replaceC1_02, String replaceC3, String replaceC4, String year, String month) {

        groupDataSyncModels.forEach(item -> {
            String money = processMoney(item.getMoney());
            item.setMoney(money);
            item.setC4("Interadj");
            item.setPlugAccount("plug19数量");
        });
        insertData.addAll(groupDataSyncModels);
        List<GroupDataSyncModel> data = groupDataSyncModels.stream()
                .filter(e -> c1.equals(e.getC1()) && c3.equals(e.getC3()) && account.equals(e.getAccount()))
                .collect(Collectors.toList());
        for (GroupDataSyncModel groupDataSyncModel : data) {
            GroupDataSyncModel groupDataSyncModel1 = new GroupDataSyncModel();
            BeanUtils.copyProperties(groupDataSyncModel, groupDataSyncModel1);
            groupDataSyncModel1.setAccount(replaceAcc);
            groupDataSyncModel1.setC1(replaceC1);
            groupDataSyncModel1.setC3(replaceC3);
            groupDataSyncModel1.setC4(replaceC4);
            groupDataSyncModel1.setMoney2(groupDataSyncModel.getMoney());
            groupDataSyncModel1.setMoney(null);
            groupDataSyncModel1.setPlugAccount("plug19数量");
            insertData.add(groupDataSyncModel1);
        }
        if (c1_02 != null) {
            List<GroupDataSyncModel> data2 = groupDataSyncModels.stream()
                    .filter(e -> c1_02.equals(e.getC1()) && c3_02.equals(e.getC3()) && account.equals(e.getAccount()))
                    .collect(Collectors.toList());
            for (GroupDataSyncModel groupDataSyncModel : data2) {
                GroupDataSyncModel groupDataSyncModel1 = new GroupDataSyncModel();
                BeanUtils.copyProperties(groupDataSyncModel, groupDataSyncModel1);
                groupDataSyncModel1.setAccount(replaceAcc2);
                groupDataSyncModel1.setC1(replaceC1_02);
                groupDataSyncModel1.setC3(replaceC3);
                groupDataSyncModel1.setC4(replaceC4);
                groupDataSyncModel1.setMoney2(groupDataSyncModel.getMoney());
                groupDataSyncModel1.setMoney(null);
                groupDataSyncModel1.setPlugAccount("plug19数量");
                insertData.add(groupDataSyncModel1);
            }
        }
        GroupDataSyncModel emptyModel = new GroupDataSyncModel();
        emptyModel.setYear(year);
        emptyModel.setMonth(month);
        emptyModel.setEntity(groupDataSyncModels.get(0).getEntity());
        emptyModel.setICP(groupDataSyncModels.get(0).getICP());
        emptyModel.setPlugAccount("plug19数量");
        insertData.add(emptyModel);
    }

    // 处理 money 字符串，反转符号并返回
    public static String processMoney(String money) {
        if (money != null && !money.isEmpty()) {
            try {
                // 将 money 字符串转换为 double，并反转其正负号
                double value = Double.parseDouble(money);
                value = -value;  // 改变正负号
                return String.valueOf(value);
            } catch (NumberFormatException e) {
                return money; // 如果格式错误，返回原始字符串
            }
        }
        return money; // 如果 money 为 null 或空，直接返回
    }
}
